!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!          The Lorenz equations: Heteroclinic orbits
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
! 
  SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
! ---------- ---- 

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)
  DOUBLE PRECISION x, y, z, rho, beta, sigma, T
  
    rho  = PAR(1)
    beta = PAR(2)
    sigma= PAR(3)
    T    = PAR(11)

    x = U(1)
    y = U(2)
    z = U(3)
  
    F(1)= T * sigma * (y-x)
    F(2)= T * (rho*x - y - x*z)
    F(3)= T * (x*y - beta*z)

    IF(IJAC.EQ.0)RETURN

    DFDU(1,1) =-T*sigma
    DFDU(1,2) = T*sigma
    DFDU(1,3) = 0.

    DFDU(2,1) = T*(rho - z)
    DFDU(2,2) =-T
    DFDU(2,3) =-T*x

    DFDU(3,1) = T*y
    DFDU(3,2) = T*x
    DFDU(3,3) =-T*beta
 
    IF(IJAC.EQ.1)RETURN

    DFDP(2,1) = T*x
    DFDP(3,2) =-T*z
    DFDP(1,3) = T*(y-x)

    DFDP(1,11) = sigma * (y-x)
    DFDP(2,11) = rho*x - y - x*z
    DFDP(3,11) = x*y -  beta*z

  RETURN 
  END 
!---------------------------------------------------------------------- 
  SUBROUTINE STPNT(NDIM,U,PAR,time) 
! ---------- ----- 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: time

  INTEGER NDM
  PARAMETER (NDM=3)
  DOUBLE PRECISION USP(NDM), V0(NDM), W0(NDM), W1(NDM)
  DOUBLE PRECISION rho, beta, sigma, eps, T, L, nrm1, dp1, zp0
  INTEGER i

    rho   = 28.d0
    beta  = 8.d0/3.d0 
    sigma = 10.d0

    PAR(1)  = rho
    PAR(2)  = beta
    PAR(3)  = sigma

!   Get the nonzero equilibrium and the needed eigenvectors:
    CALL GETDATA(NDIM,PAR,USP,V0,W0,W1)

    eps = 1.0d0  ! re-initialized in the script file het.auto
    T   = 1d-3
    L   = 1d-2

!   Set initial approximate (constant) solution (for small T)
    DO i=1,NDIM
      U(i)= USP(i) + eps * V0(i)
    ENDDO

!   Initialize the norm of the end point of the orbit
!   (which here equals the norm of the starting point):
    nrm1 = DSQRT(U(1)**2 + U(2)**2 + U(3)**2)

!   Initialize z'(0)  (not scaled):
    zp0 = U(1)*U(2) -  beta*U(3)

!   Initialize the dot product at the end of the orbit:
    dp1 = W1(1)*U(1) + W1(2)*U(2) + W1(3)*U(3)

    PAR(4)  = eps
    PAR(5)  = nrm1
    PAR(6)  = zp0
    PAR(9)  = dp1
    PAR(11) = T
    PAR(12) = L

  RETURN 
  END 
!---------------------------------------------------------------------- 
  SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
! ---------- ---- 
! 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
  DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
  DOUBLE PRECISION, INTENT(OUT) :: FB(NBC), DBC(NBC,*)

  DOUBLE PRECISION rho, beta, sigma, eps, nrm1, zp0, dp1, e2, e5
  INTEGER NDM
  PARAMETER (NDM=3)
  DOUBLE PRECISION USP(NDM), V0(NDM), W0(NDM), W1(NDM)

    rho  = PAR(1)
    beta = PAR(2)
    sigma= PAR(3)
    eps  = PAR(4)
    nrm1 = PAR(5)
    zp0  = PAR(6)
    dp1  = PAR(9)
 
    CALL GETDATA(NDIM,PAR,USP,V0,W0,W1)

    e2    = DSQRT( (U0(1)-USP(1))**2  + (U0(2)-USP(2))**2 + (U0(3)-USP(3))**2 )
    e5    = DSQRT(U1(1)**2 + U1(2)**2 + U1(3)**2)

    FB(1) = (U0(1)-USP(1))*W0(1) + (U0(2)-USP(2))*W0(2) + (U0(3)-USP(3))*W0(3)
    FB(2) = e2 - eps
    FB(3) = U0(1)*U0(2) -  beta*U0(3) - zp0
    FB(4) = W1(1)*U1(1) + W1(2)*U1(2) + W1(3)*U1(3) - dp1
    FB(5) = e5 - nrm1

    IF(IJAC.EQ.0)RETURN

    DBC(1,1) = W0(1)
    DBC(1,2) = W0(2)
    DBC(1,3) = W0(3)

    DBC(2,1) = (U0(1)-USP(1))/e2
    DBC(2,2) = (U0(2)-USP(2))/e2
    DBC(2,3) = (U0(3)-USP(3))/e2
    DBC(2,10) =-1.d0

    DBC(3,1) = U0(2)
    DBC(3,2) = U0(1)
    DBC(3,3) =-PAR(2)
    DBC(3,12) =-1.d0

    DBC(4,4) = W1(1)
    DBC(4,5) = W1(2)
    DBC(4,6) = W1(3)
    DBC(4,15) =-1.d0

    DBC(5,4) = U1(1)/e5
    DBC(5,5) = U1(2)/e5
    DBC(5,6) = U1(3)/e5
    DBC(5,11) =-1.d0

  RETURN 
  END 
!---------------------------------------------------------------------- 
  SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT) 
! ---------- ---- 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, ICP(*), NINT, IJAC
  DOUBLE PRECISION, INTENT(IN) :: PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
  DOUBLE PRECISION, INTENT(OUT) :: FI(NINT)
  DOUBLE PRECISION, INTENT(INOUT) :: DINT(NINT,*)
 
  INTEGER NDM
  PARAMETER (NDM=3)
  DOUBLE PRECISION FF(NDM), DFDU, DFDP, e1, rho, beta, sigma, T, L, x, y, z
 
    CALL FUNC(NDIM,U,ICP,PAR,0,FF,DFDU,DFDP) 

    rho  = PAR(1)
    beta = PAR(2)
    sigma= PAR(3)
    T    = PAR(11)
    L    = PAR(12)

    e1 = DSQRT(FF(1)**2 + FF(2)**2 + FF(3)**2 )
    FI(1)= e1 - L
 
    IF(IJAC.EQ.0)RETURN

    x = U(1)
    y = U(2)
    z = U(3)
  
    DINT(1,1) = T * (-FF(1)*sigma + FF(2)*(rho-z) + FF(3)*y    )/e1
    DINT(1,2) = T * ( FF(1)*sigma - FF(2)         + FF(3)*x    )/e1
    DINT(1,3) = T * (             - FF(2)*x       - FF(3)*beta )/e1
    DINT(1,14)=     ( FF(1)**2    + FF(2)**2      + FF(3)**2 )/(T*e1)
    DINT(1,15)=-1.d0 

  RETURN 
  END  
!----------------------------------------------------------------------
  SUBROUTINE GETDATA(NDIM,PAR,USP,V0,W0,W1) 
! ---------- -------
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: USP(NDIM), V0(NDIM), W0(NDIM), W1(NDIM)
  DOUBLE PRECISION rho, beta, sigma, c, ss

  INTEGER NDM, first, i, j
  PARAMETER (NDM=3)
  DOUBLE PRECISION A(NDM,NDM), WR(NDM), WI(NDM), Z(NDM,NDM), FV1(NDM)
  INTEGER IV1(NDM), IER
  COMMON /IFirst/ first

   rho  = PAR(1)
   beta = PAR(2)
   sigma= PAR(3)

!   The nonzero stationary point
!   ----------------------------
    USP(1) = DSQRT(beta*(rho - 1))  
    USP(2) = USP(1)
    USP(3) = rho - 1

!   The Jacobian at the nonzero stationary point
!   --------------------------------------------
    A(1,1) =-sigma
    A(1,2) = sigma
    A(1,3) = 0

    A(2,1) = rho - USP(3)
    A(2,2) =-1
    A(2,3) =-USP(1)

    A(3,1) = USP(2)
    A(3,2) = USP(1)
    A(3,3) =-beta

    CALL RG(NDM,NDM,A,WR,WI,1,Z,IV1,FV1,IER)

!   This is somewhat dangerous:
    c = (1-Z(3,3))/Z(3,2)
    V0(1) = c*Z(1,2) + Z(1,3)   
    V0(2) = c*Z(2,2) + Z(2,3)
    V0(3) = c*Z(3,2) + Z(3,3)

! Normalize:
    ss=DSQRT(V0(1)**2 + V0(2)**2 + V0(3)**2)
    V0(1)=V0(1)/ss
    V0(2)=V0(2)/ss
    V0(3)=V0(3)/ss
 
!   Diagnostic output
    IF(first.NE.1234)THEN
      WRITE(9,100)"Real part:"
      WRITE(9,101)(WR(i),i=1,NDM)
      WRITE(9,100)"Imaginary part:"
      WRITE(9,101)(WI(i),i=1,NDM)
      WRITE(9,100)"Eigenvectors (by column):"
      DO i=1,3
        WRITE(9,102)(Z(i,j),j=1,NDM)
      ENDDO
      WRITE(9,100)"USP:"
      WRITE(9,102)(USP(i),i=1,NDM)
      WRITE(9,100)"V0:"
      WRITE(9,102)(V0(i),i=1,NDM)
    ENDIF

!   The adjoint at the nonzero stationary point
!   -------------------------------------------
    A(1,1) =-sigma
    A(2,1) = sigma
    A(3,1) = 0

    A(1,2) = rho - USP(3)
    A(2,2) =-1
    A(3,2) =-USP(1)

    A(1,3) = USP(2)
    A(2,3) = USP(1)
    A(3,3) =-beta

    CALL RG(NDM,NDM,A,WR,WI,1,Z,IV1,FV1,IER)

!   This is somewhat dangerous:
    W0(1) = Z(1,1)
    W0(2) = Z(2,1)
    W0(3) = Z(3,1) 

!   Normalize:
    ss=DSQRT(W0(1)**2 + W0(2)**2 + W0(3)**2)
    W0(1)=W0(1)/ss
    W0(2)=W0(2)/ss
    W0(3)=W0(3)/ss

! Diagnostic output
    IF(first.NE.1234)THEN
      first=1234
      WRITE(9,100)"Adjoint: Real part:"
      WRITE(9,101)(WR(i),i=1,NDM)
      WRITE(9,100)"Adjoint: Imaginary part:"
      WRITE(9,101)(WI(i),i=1,NDM)
      WRITE(9,100)"Adjoint eigenvectors (by column):"
      DO i=1,NDM
        WRITE(9,102)(Z(i,j),j=1,NDM)
      ENDDO
      WRITE(9,100)"W0:"
      WRITE(9,102)(W0(i),i=1,NDM)
    ENDIF
100 FORMAT(/,1X,A)
101 FORMAT(1X,1P3E16.7)
102 FORMAT(1X,1P3E16.7)

!   A vector orthogonal to the 2D stable eigenspace of the origin
!   -------------------------------------------------------------
    W1(1) =-1.
    W1(2) = (1 - sigma - DSQRT((sigma-1)**2+4*rho*sigma))/(2*rho)
    W1(3) = 0.

!   Normalize:
    ss=DSQRT(W1(1)**2 + W1(2)**2 + W1(3)**2)
    W1(1)=W1(1)/ss
    W1(2)=W1(2)/ss
    W1(3)=W1(3)/ss

  RETURN 
  END  
!---------------------------------------------------------------------- 
!----------------------------------------------------------------------
  SUBROUTINE PVLS
  RETURN
  END
 
  SUBROUTINE FOPT 
  RETURN 
  END 
!----------------------------------------------------------------------
!----------------------------------------------------------------------
