!----------------------------------------------------------------------
!----------------------------------------------------------------------
      INCLUDE "equations.f90"
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, IJAC, ICP(*)
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM), DFDU(NDIM,*), DFDP(NDIM,*)

      INTEGER NDM,i,j
      PARAMETER(NDM=3)
      DOUBLE PRECISION FF(NDM), DF(NDM,NDM), sm, T

!       Get the RHS of the dynamical systems:
        CALL GETRHS(NDM,U,PAR,FF)

        T = PAR(11)
        DO i=1,NDM
          F(i) = T*FF(i)
        ENDDO

!       Get the Jacobian:
        CALL GETJAC(NDM,U,PAR,DF)

!       Set up the Floquet equations:
        sm=PAR(21)
        DO i=1,NDM
          F(NDM+i) = 0.d0
          DO j=1,NDM
            F(NDM+i) = F(NDM+i) + DF(i,j)*U(NDM+j)
          ENDDO
          F(NDM+i) = T*F(NDM+i) - sm*U(NDM+i)
        ENDDO

      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, NBC, IJAC, ICP(*)
      DOUBLE PRECISION, INTENT(IN) :: U0(NDIM), U1(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: FB(NBC), DBC(NBC,*)
      INTEGER i

!       Boundary conditions:
        DO i=1,NDIM
          FB(i) = U0(i) - U1(i)
        ENDDO

      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

      SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), NINT, IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM)
      DOUBLE PRECISION, INTENT(IN) :: UPOLD(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: FI(NINT), DINT(NINT,*)
      INTEGER NDM, i
      PARAMETER(NDM=3)

!       Integral phase condition:
        FI(1) = 0.
        DO i=1,NDM 
          FI(1) = FI(1) + U(i)*UPOLD(i)
        ENDDO

!       Integral Floquet eigenfunction normalization
        FI(2) = -PAR(22)
        DO i=NDM+1,NDIM
          FI(2) = FI(2) + U(i)**2
        ENDDO

      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
      SUBROUTINE PVLS
      RETURN
      END 

      SUBROUTINE STPNT
      RETURN 
      END 

      SUBROUTINE FOPT 
      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
