!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!          The Lorenz Equations
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 
      SUBROUTINE GETRHS(NDIM,U,PAR,F) 
!     ---------- ------
 
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION rho, beta, sigma, x, y, z

! Define the RHS of the dynamical system

        rho   = PAR(1)
        beta  = PAR(2)
        sigma = PAR(3)

        x = U(1)
        y = U(2)
        z = U(3)

        F(1) = sigma*(y-x)
        F(2) = rho*x - y - x*z
        F(3) = x*y -  beta*z

      RETURN
      END
!---------------------------------------------------------------------- 

      SUBROUTINE GETJAC(NDIM,U,PAR,DFDU) 
!     ---------- ------
 
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: DFDU(NDIM,*)
      DOUBLE PRECISION rho, beta, sigma, x, y, z

! Define the Jacobian

        rho   =  PAR(1)
        beta  = PAR(2)
        sigma = PAR(3)

        x = U(1)
        y = U(2)
        z = U(3)

        DFDU(1,1) =-sigma
        DFDU(1,2) = sigma
        DFDU(1,3) = 0.

        DFDU(2,1) = rho - z
        DFDU(2,2) =-1.d0
        DFDU(2,3) =-x

        DFDU(3,1) = y
        DFDU(3,2) = x
        DFDU(3,3) =-beta

      RETURN
      END
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
