!---------------------------------------------------------------------- 
!----------------------------------------------------------------------
      INCLUDE "equations.f90"	
!---------------------------------------------------------------------- 
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ---- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, IJAC, ICP(*)
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM), DFDU(NDIM,*), DFDP(NDIM,*)
      DOUBLE PRECISION T
      INTEGER i

!       Get the RHS of the dynamical systems
        CALL GETRHS(NDIM,U,PAR,F)

!       Scale
        T = PAR(48)
        DO i=1,NDIM
          F(i) = T*F(i)
        ENDDO
 
      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, NBC, IJAC, ICP(*)
      DOUBLE PRECISION, INTENT(IN) :: U0(NDIM), U1(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: FB(NBC), DBC(NBC,*)
      DOUBLE PRECISION rho, beta, sigma, eps, x, y, z
      DOUBLE PRECISION vx, vy, vz, xn, yn, zn, rn, rv
      DOUBLE PRECISION Tnom, T, Rnom, R
      INTEGER i

        rho  = PAR(1)
        beta = PAR(2)
        sigma= PAR(3)

!       Initial conditions for the orbit in the stable manifold:
        eps=PAR(40)
        DO i=1,NDIM
          FB(i) = U0(i) - ( PAR(23+i) + eps*PAR(26+i) )
        ENDDO

!       The coordinates of the end point of the orbit in the stable manifold:
        x = U1(1)
        y = U1(2)
        z = U1(3)

!       Keep track of the end points in PAR(41-43)
        FB(4) = x - PAR(41)
        FB(5) = y - PAR(42)
        FB(6) = z - PAR(43)

!       If PAR(44) is zero then the end point lies in the plane z = rho-1:
        FB(7) = z - (rho-1.d0) - PAR(44)

!       Retraction constraint on orbits:
        Tnom = PAR(13)
        T    = PAR(48)
        Rnom = PAR(14)
        R    = PAR(46)
        FB(8) =  (T-Tnom)*(Rnom-R)  - PAR(45)

!       The distance of the end point to the center of the sphere:
        FB(9) =  DSQRT(x**2 + y**2 + (z-rho+1.d0)**2)  - R

!       Orthogonality of end points on the sphere to the vector field:
        vx = sigma*(y-x)
        vy = rho*x - y - x*z
        vz = x*y -  beta*z
        xn=x
        yn=y
        zn= z - rho + 1
        rn= SQRT(xn**2 + yn**2 + zn**2)
        xn=xn/rn
        yn=yn/rn
        zn=zn/rn
        rv= SQRT(vx**2 + vy**2 + vz**2)
        vx = vx/rv
        vy = vy/rv
        vz = vz/rv
        FB(10) = xn*vx + yn*vy + zn*vz - PAR(47)

      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
      SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, NINT, IJAC, ICP(*)
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM)
      DOUBLE PRECISION UPOLD(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: FI(NINT), DINT(NINT,*)
      INTEGER NDM, i
      PARAMETER (NDM=3)
      DOUBLE PRECISION Q, T, FF(NDM)

!       Length of the orbit in the stable manifold

        CALL GETRHS(NDIM,U,PAR,FF)

        T=PAR(48)
        Q=0.d0
        DO i=1,NDM
          Q = Q + (T*FF(i))**2
        ENDDO
        FI(1) = DSQRT(Q) - PAR(49)

      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
      SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

!       PAR(46) is the "radius" of the current orbit's end point
!       PAR(14) is the wanted "radius", as set in the Python script
        PAR(18) = PAR(46) - PAR(14)

!       PAR(40) is eps; the step into the manifold
!       PAR(15) is eps_max; the maximum value allowed for |eps|
        PAR(19) = DABS(PAR(40)) - PAR(15)

      RETURN
      END
!----------------------------------------------------------------------
!---------------------------------------------------------------------- 
      SUBROUTINE STPNT
      RETURN 
      END 
!---------------------------------------------------------------------- 
      SUBROUTINE FOPT 
      RETURN 
      END 
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
